<?php

/**
 * @version     $Id: export.php 10 2014-10-09 08:29:05Z david.dorchies $
 * @package     Joomla16.Tutorials
 * @subpackage  Components
 * @copyright   Copyright (C) 2005 - 2010 Open Source Matters, Inc. All rights reserved.
 * @author      Christophe Demko
 * @link        http://joomlacode.org/gf/project/umrmanagement_1_6/
 * @license     License GNU General Public License version 2 or later
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// import the Joomla modellist library
jimport('joomla.application.component.modellist');

/**
 * UmrManagement Export Model
 */
class UmrManagementModelExport extends JModelList
{

    protected $categorie=null;
    protected $fieldnames;
    protected $form;
    protected $formdata=null;

    /**
     * Method to get the record form.
     *
     * @param   array   $data       Data for the form.
     * @param   boolean $loadData   True if the form is to load its own data (default case), false if not.
     * @return  mixed   A JForm object on success, false on failure
     * @since   1.6
     */
    public function getForm($data = array(), $loadData = false)
    {
        // Get the form.
        JForm::addFormPath(JPATH_COMPONENT . '/models/forms');
        JForm::addFieldPath(JPATH_COMPONENT . '/models/fields');
        $form = JForm::getInstance('com_umrmanagement.export', 'export', array('control' => 'jform'));
        if (empty($form))
        {
            return false;
        }
        // Inject previous post of the data
        $data  = JRequest::getVar('jform', array(), 'post', 'array');
        // Load the data into the form after the plugins have operated.
        if($data) {
            $form->bind($data);
        }
        return $form;
    }


    /**
     * Récupère les données du formulaire export
     * @author David Dorchies
     * @date 29/08/2014
     */
    public function getFormData () {
        if(is_null($this->formdata)) {
            $this->formdata  = JRequest::getVar('jform', array(), 'post', 'array');
        }
        return $this->formdata;
    }


    /**
     * Récupère la catégorie sélectionnée dans le formulaire export
     * @author David Dorchies
     * @date 25/08/2014
     */
    public function getCategorie () {
        if(is_null($this->categorie)) {
            $data  = $this->getFormData();
            if(isset($data['categorie'])) {
                $this->categorie = $data['categorie'];
            }
        }
        return $this->categorie;
    }


    /**
     * Method to build an SQL query to load the list data.
     *
     * @return  string  An SQL query
     */
    protected function getListQuery()
    {
        // Create a new query object.
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);

        // Liste des champs des tables umrmanagement et umrmanagement_mouvement à exporter
        $this->fieldnames = array('name','sexe','birth_year','etablissement','grade','axe','etablissement');
        $select = array_merge(array('idmvt','categorie'),$this->fieldnames);
        if(!$this->getCategorie()) {
            // Pour le choix toutes catégories, on ajoute une première colonne catégorie
            $this->fieldnames = array_merge(array('categorie'),$this->fieldnames);
        } else {
            $query->where($db->quotename('categorie') . ' = ' . $this->getCategorie());
        }
        $data = $this->getFormData();
        if(!isset($data['aladate'])) {
            $data['aladate'] = date('Y-m-d');
        }
        if($this->getCategorie() < 7) {
            // Toutes les catégories ou personnels permanents
            $query->where('DATEDIFF('.$db->quotename('staff_in').',\''.$data['aladate'].'\')<=0 AND ('.$db->quotename('DepartInconnu').'=1 OR DATEDIFF(\''.$data['aladate'].'\', '.$db->quotename('staff_out').')<=0)');
        }
        else {
            // Personnels temporaires
            $query->where('DATEDIFF(\''.$data['fin'].'\','.$db->quotename('staff_in').')>=0 AND ('.$db->quotename('DepartInconnu').'=1
            OR DATEDIFF('.$db->quotename('staff_out').',\''.$data['debut'].'\')>=0)
            AND DATEDIFF('.$db->quotename('staff_out').','.$db->quotename('staff_in').')>='.$data['duree']);
        }
        $query
            ->select($db->quotename($select))
            ->from($db->quotename('#__umrmanagement'))
            ->join('INNER', $db->quoteName('#__umrmanagement_mouvement') . ' USING (' . $db->quoteName('id') . ')')
            ->order($db->quotename('name'));

        return $query;
    }

    /**
     * Récupère la liste des colonnes à afficher dans le tableau de sortie
     * @author David Dorchies
     * @date 25/08/2014
     * @return Liste du nom des champs à afficher
     * @note La propriété fieldnames est renseignée par $this->getQueryList()
     */
    public function getFieldNames() {
        return $this->fieldnames;
    }

    /**
     * Get the list
     *
     * @return  mixed  An array of data items on success, false on failure.
     *
     * @since   1.6
     */
    public function getItems()
    {
        $items = parent::getItems();

        // Bail out on an error or empty list.
        if (empty($items))
        {
            return $items;
        }
        // Add Label to key fields
        require_once(JPATH_COMPONENT_SITE.DS.'models'.DS.'umrtools.php');
        foreach ($items as &$item)
        {
            $item->etablissement = UmrManagementModelUmrTools::getLibelleCourt('etablissement',$item->etablissement);
            $item->grade = UmrManagementModelUmrTools::getLibelleCourt('Grade',$item->grade,$item->categorie);
            $item->axe = UmrManagementModelUmrTools::getLibelleCourt('Axe',$item->axe);
            $item->id_categorie = $item->categorie;
            $item->categorie = UmrManagementModelUmrTools::getLibelleCourt('Categorie',$item->categorie);
        }

        //***************************************
        // Traitement des champs complémentaires
        //***************************************
        // Récupération de la liste des champs
        $aExtraChamps = UmrManagementModelUmrTools::getChampsExtraInfo($this->getCategorie ());
        // Mise à jour des noms de champ des colonnes
        $this->fieldnames = array_merge($this->fieldnames,$aExtraChamps);
        foreach ($items as &$item) {
            $aExtraItem = UmrManagementModelUmrTools::getChampsExtraItem($item->idmvt);
            foreach($aExtraChamps as $sChamp) {
                if(isset($aExtraItem[$sChamp])) {
                    $aExtraParam = UmrManagementModelUmrTools::getChampsExtraParam($sChamp);
                    switch($aExtraParam['type']) {
                        case 'umrlibelle':
                            $item->$sChamp = UmrManagementModelUmrTools::getLibelleCourt($sChamp,$aExtraItem[$sChamp]);
                            break;
                        default:
                            $item->$sChamp = $aExtraItem[$sChamp];
                    }
                }
            }
        }
        return $items;
    }


    /**
     * Method to auto-populate the model state.
     *
     * This method should only be called once per instantiation and is designed
     * to be called on the first call to the getState() method unless the model
     * configuration flag to ignore the request is set.
     *
     * @return  void
     *
     * @note    Calling getState in this method will result in recursion.
     * @since   11.1
     */
    protected function populateState($ordering = NULL, $direction = NULL)
    {
        parent::populateState();

        if(JRequest::getWord('format')=='raw') {
            $this->setState('list.start', 0);
            $this->setState('list.limit', 0);
        }

    }

}
